//Path sampling for marg. density
//MIDPOINT VERSION
//
//Written by Lou Mariano (2002)
//Base MCMC code from hrm.cpp
//
//g++ -O2 -o PATH.exe PATH.cpp PATH.update.cpp hrm.pcm.cpp rngen.cpp
#include<math.h>
#include<stdlib.h>
#include<iostream.h>
#include<fstream.h>
#include<iomanip.h>
#include "PCM.hpp"
#include <time.h>

#define drnunf() drand48()
#define drnnof() rnorm();

void cumsum(int dim,double gamma[],double eta[]); //in hrm.pcm
double rnorm(void);

void update_gamma(int nresp,Quest &task,double theta[],int x[]
		  ,double b, double m[], double v[]); //in update
void update_var(int nresp, double theta[], double &mu,double &tau,
		double b, double pmumean, double pmuvar,
		double ptaumean, double ptauvar); //in update
void update_theta(int nquest,Quest task[],double &theta,
		  double mu,double tau,int resp[]
		  ,double b, double m, double v); //in update
void update_lat_resp(Quest task,double theta,Mat<double> tran[],
		     int sc[],int nrate,int &xi);
void calc_trans(int ncat,double mu,double sigma,
		Mat<double> &trans); //in update
void update_rater(Quest item[],int nresp,int nquest,//in update
                  Mat<int> x,Mat<int> xi,double &shift,double &scale,
		  Mat<double> &trans,double cshvar,double cscvar
		  ,double b, double msh, double vsh, double msc, double vsc); 
void update_beta_cons(int nresp,Quest &task,Quest &taskc
		      ,double theta[],int x[],int xc[]
		      ,double b, double m, double v); //in update

double prob_calc(Quest task,double theta,int x);
double norm_dens(double x, double mean, double var);
double lognorm_dens(double x, double mean, double sd);
void pcm (Quest task,double theta,double prob[]);

main(int argc,char *argv[]){
  if(argc < 8+atoi(argv[3])){ //checks for proper form in command line 
    cout<<"ERROR:  command-line:  hrm.exe filename nresp nquest "
	<<"ncat1 ncat2 ... ncat_nquest nrater ngrid nsim/grid nburn"
	<<endl;
    exit(99);
  }
  int maxcat=0; //will hold the maxium number of categories among all items
  int nquest=atoi(argv[3]),nsim=atoi(argv[6+nquest]),nresp=atoi(argv[2]),
    nrater=atoi(argv[4+nquest]); //declares assigns 
                                 //number of questions (nquest)
                                 //number of simulations (nsim)
                                 //number of examinees (nresp)
                                 //number of raters (nrater)
  int ngrid=atoi(argv[5+nquest]),nburn=atoi(argv[7+nquest]);
                                 //num of grid points (ngrid)
                                 //num of burn-in per grid (nburn)
  int nsimpb=nsim-nburn;//number of sims per grid actually used for the estimate

  int bncat[nquest]; //will hold number of categories for each question
  for(int i=0;i<nquest;i++){
    bncat[i]=atoi(argv[4+i]); //assign number of categories
    maxcat = bncat[i] > maxcat ? bncat[i] : maxcat;
  } //calculate ans assign maxium number of categories

  char *infile= argv[1]; //assign file containing the data

  //ESTABLISH TIME...Start and End time and total minutes of run time
  //                 be written to the log.out file
  time_t time1, time2;  //declaration of time variables
  time(&time1);
  
  //confirm command line input back to the screen
  cout<<"Data in "<<argv[1]<<endl;
  cout<<nresp<<" Responses on"<<endl;
  cout<<nquest<<" Questions by"<<endl;
  cout<<nrater<<" Raters scored"<<endl;
  for(int i=0;i<nquest;i++)
    cout<<"0-"<<bncat[i]-1<<endl;
  cout<<nsim<<" Simulations"<<endl;

  /////////////////////////////SET SEED//////////////////////
  unsigned short int theseeds[] = { 25 , 35, 45 };
  seed48(theseeds);//set the seed for drand48
  /////////////////////////////SET SEED//////////////////////

  char *outlog="output/log.out";
  char *outu="output/uhat.out";

  ofstream oUHat(outu,ios::out);
  ofstream oLog(outlog,ios::out);
  ifstream iFile(infile,ios::in); //establish input file
  ifstream iAveFile("/TOIL-U2/ltm/MAIN/CODE/ANNEAL/outputave/ave.out"); 
  //to read in post aves.
  
  ifstream iVarFile("/TOIL-U2/ltm/MAIN/CODE/ANNEAL/outputvar/var.out"); 
  //to read in post vars.
  
  fstream oInit;
  
  //ESTABLISH TIME...continued
  oLog <<setprecision(8);
  oLog <<"Program started at "<< ctime(&time1) << endl;

  //confirm command line input to the log file
  oLog<<"Data in "<<argv[1]<<endl;
  oLog<<nresp<<" Responses on"<<endl;
  oLog<<nquest<<" Questions by"<<endl;
  oLog<<nrater<<" Raters scored"<<endl;
  for(int i=0;i<nquest;i++)
    oLog<<"0-"<<bncat[i]-1<<endl;
  oLog<<ngrid<<" Gridpoints"<<endl;
  oLog<<nsim<<" Simulations per grid"<<endl;
  oLog<<"burn-in of "<<nburn<<endl<<endl;


  //Set up gridpoints

  oLog << "Gridpoints:  " << endl;
  double t[ngrid];
  double gg;
  double ng=static_cast<double>(ngrid);
  for (int g=0; g<ngrid; g++){
    gg=static_cast<double>(g);
    t[g]= ( ( 2 * gg ) + 1 )/( 2 * ng );
    oLog << t[g] << "  ";
  }

  //some alternative gridpoint options
  //    for (int g=0; g<ngrid; g++){
  //      t[g]= static_cast<double>(g)/(static_cast<double>(ngrid) - 1);
  //      oLog << t[g] << "  ";
  //    }

  //  double kk= exp(log(exp(1)-1)/log(exp(1)));
  //  for (int g=0; g<ngrid; g++){
  //    t[g]= (exp(static_cast<double>(g)/(ngrid-1))-1)/kk;
  //    oLog << t[g] << "  ";
  //  }

  //  double kk= log( 9 ) / log( 10 );
  //  for (int g=0; g<ngrid; g++){
  //    t[g]= ( pow( 10, static_cast<double>(g)/(ngrid-1) )-1)/pow( 10, kk);
  //    oLog << t[g] << "  ";
  //  }

  oLog << endl << endl;

  oUHat << ngrid << "  " << nsimpb << endl;


  int tmp; //will be used as a temporary holder for input from the datafile

  int vec[nrater];//will be a placeholder counting device for the raters
  
  Resp subj[nresp]; //vector declares a variable of class Resp for each 
                    //respondent (i.e. examinee) which will hold info unique
                    //to the respondent (see PCM.hpp for object detail) 

  Quest item[nquest]; //vector decalres a variable of class Quest for each
                      //item which will hold info unique to each item
                      //(see PCM.hpp for object detail) 
  

  for(int i=0;i<nresp;i++){
    subj[i].SetPar(nquest,nrater);  
    //SetPar in object Quest establishes
    //an int vector "raters" of length nrater 
    //an int vector "lat_resp" of lenghth nquest
    //a matrix "score" of dim nquest x nrater
    //"nquest" (same as above)
    //"nrater" (same as above)

    for(int j=0;j<nrater;j++){//here vec becomes a vector of integers 
      vec[j]=j;               //from 0 to nrater-1
    }

    //set up the scores for each Resp.
    for(int j=0;j<nquest;j++){       
      for(int r=0;r<nrater;r++){
	iFile>>tmp;               //read in rater r's score of Resp i's 
	                          //response to item j
	subj[i].SetScore(j,r,tmp);//and assign it to Resp's score matrix in    
      }                           //cell (j,r)    
    }
    subj[i].SetRater(nrater,vec); //assigns Resp.raters[] as a vector
  }                               //of integers from 0 to nrater-1
  
  //Initial Values-----------------------------------------------------
  
  
  //mu and tau are the examinee population mean and variance for ability
  //and will hold the current iteration
  double mu=0,tau=5,shift[nrater],scale[nrater];
  //shift and scale will the current rater severity (shift) and 
  //consistency (scale) for the current iteration

  double theta[nresp];//will hold current respondent ability iteration
  
  int task[nresp],taskc[nresp],num[nrater];
  //when updating the item parameters (w/ sum-to-0 constraint) task and taskc 
  //are used as temporary holders of the ideal scores of each Resp for the 
  //current item being updated (task) and the final constrained item (taskc)
  
  Mat<double> trans[nrater]; //will hold unnormalized transition probabilities
  //i.e. P[rater r rates (i,j) as category k | ideal score l ] is 
  //proportional to trans[r].GetElem(l,k)
  //NOTE: These MUST be scaled to add to one to be real probabilities
 
  Mat<int> xi; //matrix which will hold curent iteration's ideal scores (xi_ij)
  
  xi.Mat2(nresp,nquest); //establish dimentions of ideal score matrix as 
                         //nresp x nquest
  
  for(int r=0;r<nrater;r++){
    num[r]=0;//makes num[] a vector of all zeros (initializing count)
    trans[r].Mat2(maxcat,maxcat);//establish the dim of the rater prob. matrix
    //as maxcat x maxcat.  For items where the number of categories is fewer 
    //than maxcat, this works fine when the scaling is done over the actual 
    //set of categories for that item.
  
    shift[r]=0; //initialize rater shift starting value to 0
    scale[r]=1; //initialize rater scale starting value to 1

    calc_trans(maxcat,shift[r],scale[r],trans[r]); //calculate the rater
                                   //transition prob. matrix given the current
                                   //shift & scale and store in trans[r]
  }
  
  
  for(int i=0;i<nquest;i++){ //for each item
    item[i].SetNcat(bncat[i]);//set the number of item categories
    item[i].SetBeta(0);//initialize item difficulty starting value to 0
  }
  
  for(int i=0;i<nresp;i++){ //for each Resp 
    for(int j=0;j<subj[i].nraters;j++){
      num[subj[i].raters[j]]++ ; //sets num[r] to nresp
    }
    theta[i]=drnnof();        //Random Initial Values for ability params
  }
  
  //--------------------------------------------------------------------

  
  Mat<int> sc_r[nrater]; //will be matrix of scores by rater r
  
  
  for(int r=0;r<nrater;r++){ //for each rater
    sc_r[r].Mat2(num[r],nquest); //sets dim to nresp x nquest
  }


  for(int i=0;i<nresp;i++){ //for each Resp 
    for(int j=0;j<nquest;j++){ //for each question
      for(int r=0;r<nrater;r++){//for each rater
	sc_r[r].SetElem(i,j,subj[i].score.GetElem(j,r));
      }//This makes sc_r[r] a matrix of r's score of i's response to
    }//item j, where sc_r[r](i,j) is bncat[j] if (i,j) not scored by r
  }



  //_______________________________________________________________
  //*** Reconstruct posterior averages

  double hold;

  iAveFile>>hold; 
  double muave=hold; 
  iAveFile>>hold; 
  double tauave=hold; 
  double shiftave[nrater], scaleave[nrater];
  for (int r=0; r<nrater; r++){
    iAveFile>>hold;
    shiftave[r]=hold;
    iAveFile>>hold;
    scaleave[r]=hold;
  }

  double betaave[nquest];
  double gammaave[nquest][maxcat];
  for (int j=0; j < nquest; j++){
    iAveFile>>hold;
    betaave[j] = hold;
    for (int k=0; k < item[j].ncat-1; k++){
      iAveFile>>hold;
      gammaave[j][k] = hold;
    }
  }
  double bc=0;
  for (int j=0; j < nquest-1; j++){
    bc+=betaave[j];
  }
  double thetaave[nresp];
  for (int i=0; i<nresp; i++){
    iAveFile>>hold;
    thetaave[i]=hold;
  }

  //_______________________________________________________________
  //*** Reconstruct posterior variances

  iVarFile>>hold; 
  double muvar=hold; 
  iVarFile>>hold; 
  double tauvar=hold; 
  double shiftvar[nrater], scalevar[nrater];
  for (int r=0; r<nrater; r++){
    iVarFile>>hold;
    shiftvar[r]=hold;
    iVarFile>>hold;
    scalevar[r]=hold;
  }

  double betavar[nquest];
  double gammavar[nquest][maxcat];
  for (int j=0; j < nquest; j++){
    iVarFile>>hold;
    betavar[j] = hold;
    for (int k=0; k < item[j].ncat-1; k++){
      iVarFile>>hold;
      gammavar[j][k] = hold;
    }
  }

  double thetavar[nresp];
  for (int i=0; i<nresp; i++){
    iVarFile>>hold;
    thetavar[i]=hold;
  }

  //_______________________________________________________________
 
  double rowtmp[maxcat], norm[maxcat];
  double lndens, lnprior, lnpn;
  double pcmprob[maxcat];
  double prod, likej;

  double est[ngrid];//will hold per grid estimate
  for (int g=0; g<ngrid; g++){
    est[g]=0;
  }
  double lhat=0;//will hold final estimate

  double theder;

  //_________________________________________________________
  //Start Gridpoints
  //__________________________________________________________


  double tt;//REPLACE W/ CYCLE THROUGH t[g]

  for (int g=0; g<ngrid; g++){
    tt=t[g];

    oLog << endl << "Grid " << g+1 << endl;

    mu=muave;
    tau=tauave;

    for(int r=0;r<nrater;r++){//initialize rater shift and scale
      shift[r]=shiftave[r];
      scale[r]=scaleave[r];
      calc_trans(maxcat,shift[r],scale[r],trans[r]);
    }//and set up normalizing constants for updating latent and rater

    for(int j=0;j<nquest;j++){
      item[j].SetBeta(betaave[j]);//sets ea. item param to 0
      for(int k=0; k<item[j].ncat; k++){
	item[j].gamma[k]=gammaave[j][k];
      }
    }
  
    for(int i=0;i<nresp;i++){
      theta[i]=thetaave[i];        //Random Initial Values for Theta
    }

    //_________________________________________________________
    //Start Simulations
    //__________________________________________________________
    for(int isim=0;isim<nsim;isim++){ //for each simulation

      if ( (isim % 10) == 9){oLog << isim+1 << endl;}//every 10th iteration
      //output current iteration to the log.out file

      for(int i=0;i<nresp;i++){//for each respondent
	for(int j=0;j<nquest;j++){//for each item 
	  subj[i].score.GetRow(j,vec);//get the scores each rater gave to 
	                            //Resp i for item j and put them in vec
	  //NOTE:  when rater r does not rate (i,j) the program expects a score 
	  //equal to the number of rating categories for j.  For example, if 
	  //item j has 3 scoring categories, each cell of row 3 of 
	  //Resp.score would contain one of {0,1,2,3}, where a 0, 1 or 2 are 
	  //actual scores and a 3 indicates that the rater did not rate (i,j)    
	    
	  update_lat_resp(item[j],theta[i],trans,vec,
			  subj[i].nraters,subj[i].lat_resp[j]);
	  //updates the ideal score (xi_ij) and assigns it to Resp i's
	  //ideal score vector (subj[i].lat_resp[j])

	  xi.SetElem(i,j,subj[i].lat_resp[j]);//copy the updated ideal score 
	  //to the matrix of all ideal scores

	}
	update_theta(nquest,item,theta[i],mu,tau,subj[i].lat_resp,
		     tt,thetaave[i],thetavar[i]);
	//updates the current ability parameter theta_i

      }
      for(int j=0;j<nquest;j++){
	xi.GetColumn(j,task); //assign ideal scores of current item to task
	xi.GetColumn(nquest-1,taskc); //assign ideal scores of constrained 
	//item to taskc
	if(j != nquest-1) //if not on the final item...
	  update_beta_cons(nresp,item[j],item[nquest-1] //update current item
			   ,theta,task,taskc,tt,betaave[j],betavar[j]);  
	//parameter and adjust constrained item parameter
      
	update_gamma(nresp,item[j],theta,task,tt,gammaave[j],gammavar[j]);
	//update current item's item-step params

      }

      update_var(nresp,theta,mu,tau,tt,muave,muvar,tauave,tauvar); 
      //update population ability mean and var

      for(int r=0;r<nrater;r++){//for each rater
	update_rater(item,nresp,nquest,sc_r[r],xi, 
		     shift[r],scale[r],trans[r],.05,.1,
		     tt, shiftave[r], shiftvar[r], scaleave[r], scalevar[r]);  
	//updates current rater's shift and scale parameters
	//.05 is shift jump variance
	//.1  is scale jump variance
	//change jump var's as necessary
      
      }//ends for r...

      //***************sims end here

      //*************now calc densities

      if (isim >= nburn){

	lndens=0;//initalize log density

	for (int i=0; i<nresp; i++){

	  lndens += log( norm_dens( theta[i], mu, tau ) );

	  for (int j=0; j<nquest; j++){
	    likej=0;
	    pcm(item[j],theta[i],pcmprob);

	    for (int z=0; z<item[j].ncat; z++){
	      prod=1;

	      for (int r=0; r<nrater; r++){

		if(subj[i].score.GetElem(j,r) < item[j].ncat){
		  trans[r].GetRow( z , rowtmp );
		  cumsum(item[j].ncat,rowtmp,norm);
		  prod *= rowtmp[subj[i].score.GetElem(j,r)] / norm[item[j].ncat-1];
		}//ends if subj[...
	      }//ends for r...


	      prod *= pcmprob[z];
	      likej+= prod;

	    }//ends for z...

	    lndens += log(likej);

	  }//ends for j...
	}//ends fot i...
	//end log density

	lnprior=0;//initialize log prior

	lnprior += log( norm_dens(mu, 0, 10) );

	lnprior += ( -2 * log(tau) ) - ( 1 / tau);
	//log of IG(1,1) dens.

	for (int r=0; r<nrater; r++){
	  lnprior += log( norm_dens(shift[r], 0, 10) );
	  lnprior += log( lognorm_dens(scale[r], 0, sqrt(10)) );
	}

	for (int j=0; j<nquest-1; j++){
	  lnprior += log( norm_dens(item[j].beta, 0, 10) );
	  
	  for (int k=0; k<item[j].ncat-2; k++){
	    lnprior += log( norm_dens(item[j].gamma[k], 0, 10) );
	  }
	}
	for (int k=0; k<item[nquest-1].ncat-2; k++){
	  lnprior += log( norm_dens(item[nquest-1].gamma[k], 0, 10) );
	}

	lndens += lnprior;//lndens now prop to ln posterior dens
	//with full normalized likelihood



	lnpn=0;//initialize log pn

	lnpn += log( norm_dens(mu, muave, muvar) );

	lnpn += log( norm_dens(tau, tauave, tauvar) );

	for (int r=0; r<nrater; r++){
	  lnpn += log( norm_dens(shift[r], shiftave[r], shiftvar[r]) );
	  lnpn += log( norm_dens(scale[r], scaleave[r], scalevar[r]) );
	}

	for (int j=0; j<nquest-1; j++){
	  lnpn += log( norm_dens(item[j].beta, betaave[j], betavar[j]) );

	  for (int k=0; k<item[j].ncat-2; k++){
	    lnpn += log( norm_dens(item[j].gamma[k], gammaave[j][k], gammavar[j][k]) );
	  }
	}
	for (int k=0; k<item[nquest-1].ncat-2; k++){
	  lnpn += log( norm_dens(item[nquest-1].gamma[k], gammaave[nquest-1][k], gammavar[nquest-1][k]) );
	}

	for (int i=0; i<nresp; i++){
	  lnpn += log( norm_dens(theta[i], thetaave[i], thetavar[i]) );
	}//done calc lnpn

	oUHat << lndens - lnpn<<endl;

	est[g]+= (lndens - lnpn);//this is sum U w/in grid 

      }

    }//end simulations

    est[g] = est[g]/nsimpb;//take within grid average
    oUHat << est[g]<<endl<<endl;
    oLog << endl << "Uhat for grid " << g+1 << ":  " <<est[g] <<endl;

    lhat += est[g]/ng;
    oLog << est[g]/ng << "  " << lhat << endl;

  }//end grids

  oLog << endl;
  oLog << "ln m(x) estimate:  " << lhat << endl;

  //write ending time
  time(&time2);
  oLog <<"Program lasted for "<<difftime(time2,time1)/60.0<<" minutes and ended at "<< ctime(&time2);
  

  return(0);
}


