//Calculates the variance and std. error for 
//Path Sampling est of m(x) 
//via the method of Newey and West (1987)
//using output from PATH.cpp

//Written by Lou Mariano (2002)

//to compile:
//g++ -O2 -o PATH.var.exe PATH.var.cpp 
//
#include<math.h>
#include<stdlib.h>
#include<iostream.h>
#include<fstream.h>
#include<iomanip.h>
#include "PCM.hpp"
#include <time.h>

main(int argc,char *argv[]){

  //TRY TO ESTABLISH TIME
  time_t time1, time2;  //declaration of time variables
  time(&time1);

  int acl;//will be the lag at which the autocorrelation tapers off

  if (argc == 1){
    cout << "input autocorrelation lag     ";
    cin >> acl;
      }
  else{
    acl = atoi(argv[1]);
  }


  char *outseln="output/var.out";
  char *outlogvar="output/logvar.out";
  char *outtmp="output/tmp.out";
  ofstream oTmp(outtmp,ios::out);
  ofstream oLog(outlogvar,ios::out);
  ofstream oSL(outseln,ios::out);

  ifstream iNV("output/uhat.out");

  int tmp;
  double hld;

  iNV >> tmp;
  int ngrid=tmp;//number of gridpoints

  iNV >> tmp;
  int nsimpb=tmp;//number of simulations per gridpoint


  double t[ngrid];
  for (int g=0; g<ngrid; g++){
    t[g]= static_cast<double>(g)/(static_cast<double>(ngrid) - 1);
    oLog << t[g] << "  ";
  }

  double aa[ngrid];
  aa[0]=0;
  for (int g=1; g<ngrid; g++){
    aa[g]=t[g]-t[g-1];
  }
  for (int g=0; g<ngrid-1; g++){
    aa[g]+=t[g+1]-t[g];
  }


  double uhat[ngrid], uh[ngrid][nsimpb];

  for (int g=0; g<ngrid; g++){
    for (int s=0; s<nsimpb; s++){
      iNV >> hld;
      uh[g][s]=hld;
    }
    iNV >> hld;
    uhat[g]=hld;
  }


  for (int g=0; g<ngrid; g++){
    for (int s=0; s<nsimpb; s++){
      uh[g][s] = uh[g][s] - uhat[g];
    }
  }

  double omega[acl+1][ngrid][ngrid];
  //initialize omega
  for(int a = 0; a < acl+1; a++){
    for (int w = 0; w < ngrid; w++){
      for (int z = 0; z < ngrid; z++){
	omega[a][w][z] = 0;
      }
    }
  }//all cells set to zero before starting sum below

  //calc omegas  //a = s in paper  //acl = m in paper
  for (int a = 0; a < acl+1; a++){//calc acl+1 different omegas
    for (int h = a+1; h < nsimpb; h++){//sum_{g=s+1}^M (sum over sims) 
      for (int w = 0; w < ngrid; w++){//cycle through each row of omega_a
	for (int z = 0; z < ngrid; z++){//cycle through ea. col. of omega_a
    	  omega[a][w][z] += uh[w][h]*uh[z][h-a];
	}
      }
    }
    for (int w = 0; w < ngrid; w++){//cycle through each row of omega_a
      for (int z = 0; z < ngrid; z++){//cycle through ea. col. of omega_a
	omega[a][w][z] = omega[a][w][z]/nsimpb;
      }
    }
    oLog << a << endl;
  }


  //calc variance (regular scale)
  double uvar[ngrid][ngrid];
  double pre[acl+1];

  for (int w = 0; w < ngrid; w++){//initializing uvar @ omega_0
    for (int z = 0; z < ngrid; z++){
      uvar[w][z] = omega[0][w][z];
    }
  }

  for (int a = 0; a < acl+1; a++){
    pre[a] = 1 - ( static_cast<double>(a) / ( static_cast<double>(acl) + 1 ) );
  }

  for (int a = 1; a < acl+1; a++){
    for (int w = 0; w < ngrid; w++){//cycle through each row of uvar
      for (int z = 0; z < ngrid; z++){//cycle through ea. col. of uvar
	uvar[w][z] += pre[a] * ( omega[a][w][z] + omega[a][z][w] );
      }
    }
  }

  for (int w = 0; w < ngrid; w++){//cycle through each row of uvar
    for (int z = 0; z < ngrid; z++){//cycle through ea. col. of uvar
      uvar[w][z] = uvar[w][z]/nsimpb;
    }
  }


  double aauvar[ngrid];

  for (int g=0; g<ngrid; g++){
    aauvar[g]=0;
  }

  for (int z = 0; z < ngrid; z++){//cycle through ea. col. of uvar
    for (int w = 0; w < ngrid; w++){//cycle through each row of uvar
      aauvar[z] += aa[w] * uvar[w][z];
    }
  }

  double lhatvar=0;
  for (int g=0; g<ngrid; g++){
    lhatvar += aauvar[g] * aa[g];
  }

  oLog << "variance of m(x) estimate:  " << lhatvar << endl;
  oLog << "s.e. of m(x) estimate:  " << sqrt( lhatvar ) << endl;
  cout << "variance of m(x) estimate:  " << lhatvar << endl;
  cout << "s.e. of m(x) estimate:  " << sqrt( lhatvar ) << endl;



  //write ending time
  time(&time2);
  oLog <<"Program lasted for "<<difftime(time2,time1)/60.0<<" minutes and ended at "<< ctime(&time2);
  

  return(0);
}


