//This program fits the HRM via MCMC.  
//Originally written by Matt Johnson (1999)
//Updated by Lou Mariano (2000, 2001, 2002)

//***RNG*** on lines 27, 76, 77

//This program utilizes the files:
//hrm32.update.cpp hrm.pcm.cpp rngen32.cpp PCM.hpp mt.cpp

//To compile:
//g++ -O2 -o hrm32.exe hrm32.cpp hrm32.update.cpp hrm.pcm.cpp rngen32.cpp

//Before running this program, select which examinee's ability parameters 
//will be retained:  see line that begins "oTheta<<"
//When running, jump variances may be adjusted here for the raters, see 
//line that starts "update_rater" within main; jump vars for the others may 
//be adjusted within the update file.

#include<math.h>
#include<stdlib.h>
#include<iostream.h>
#include<fstream.h>
#include<iomanip.h>
#include "PCM.hpp"
#include <time.h>

#include "mt.cpp" //***RNG***

#define drnnof() rnorm();

void cumsum(int dim,double gamma[],double eta[]); //in pcmsign
double rnorm(void);
void update_gamma(int nresp,Quest &task,double theta[],int x[]); //in update
void update_var(int nresp, double theta[], double &mu,double &tau); //in update
void update_theta(int nquest,Quest task[],double &theta,
		  double mu,double tau,int resp[]); //in update
void update_lat_resp(Quest task,double theta,Mat<double> trans[],
		     int sc[],int nrate,int &xi); //in update
void calc_trans(int ncat,double mu,double sigma,
		Mat<double> &trans); //in update
void update_rater(Quest item[],int nresp,int nquest,
                  Mat<int> x,Mat<int> xi,double &shift,double &scale,
		  Mat<double> &trans,double cshvar,double cscvar); //in update
void update_beta_cons(int nresp,Quest &task,Quest &taskc
		 ,double theta[],int x[],int xc[]); //in update

main(int argc,char *argv[]){
  if(argc < 6+atoi(argv[3])){ //checks for proper form in command line 
    cout<<"ERROR:  command-line:  hrm.exe filename nresp nquest "
	<<"ncat1 ncat2 ... ncat_nquest nrater nsim"
	<<endl;
    exit(99);
  }
  int maxcat=0; //will hold the maxium number of categories among all items
  int nquest=atoi(argv[3]),nsim=atoi(argv[5+nquest]),nresp=atoi(argv[2]),
    nrater=atoi(argv[4+nquest]); //declares and assigns 
                                 //number of questions (nquest)
                                 //number of simulations (nsim)
                                 //number of examinees (nresp)
                                 //number of raters (nrater)

  int bncat[nquest]; //will hold number of categories for each question
  for(int i=0;i<nquest;i++){
    bncat[i]=atoi(argv[4+i]); //assign number of categories
    maxcat = bncat[i] > maxcat ? bncat[i] : maxcat;
  } //calculate and assign maxium number of categories

  char *infile= argv[1]; //assign file containing the data

  //ESTABLISH TIME...Start and End time and total minutes of run time
  //                 written to the log.out file
  time_t time1, time2;  //declaration of time variables
  time(&time1);
  
  //initialize random sumber generator ans set seed
  unsigned long int seed = 050562; //***RNG***
  init_genrand(seed);              //***RNG***




  //confirm command line input back to the screen
  cout<<"Data in "<<argv[1]<<endl;
  cout<<nresp<<" Responses on"<<endl;
  cout<<nquest<<" Questions by"<<endl;
  cout<<nrater<<" Raters scored"<<endl;
  for(int i=0;i<nquest;i++)
    cout<<"0-"<<bncat[i]-1<<endl;
  cout<<nsim<<" Simulations"<<endl;

  //declare output files
  char *outbeta="output/beta.out"; 
  //will hold item parameter iterations (in columns)
  
  char *outgamma="output/gamma.out";
  //will hold item-step parameter iterations
  //one row per item-step per iteration

  char *outmusig="output/mu_sig.out";
  //will hold population and rater params
  //ea. row:  var mean shift(1) scale(1) ... shift(nrater) scale(nrater)

  char *outtheta="output/theta.out";
  //will hold selected ability params (in columns)

  char *initval="output/inits";
  //will hold initial values used for the parameters

  char *outlog="output/log.out";
  //will hold log file tracking time, command line info, and iterations (by 10)

  ofstream oLog(outlog,ios::out);
  ofstream oTheta(outtheta,ios::out);
  ofstream oGamm(outgamma,ios::out);
  ofstream oBeta(outbeta,ios::out);
  ofstream oMuSi(outmusig,ios::out);
  ifstream iFile(infile,ios::in); //establish input file
  fstream oInit;
  
  //ESTABLISH TIME...continued
  oLog <<setprecision(3);
  oLog <<"Program started at "<< ctime(&time1) << endl;

  //confirm command line input to the log file
  oLog<<"Data in "<<argv[1]<<endl;
  oLog<<nresp<<" Responses on"<<endl;
  oLog<<nquest<<" Questions by"<<endl;
  oLog<<nrater<<" Raters scored"<<endl;
  for(int i=0;i<nquest;i++)
    oLog<<"0-"<<bncat[i]-1<<endl;
  oLog<<nsim<<" Simulations"<<endl;

  int tmp; //will be used as a temporary holder for input from the datafile

  int vec[nrater];//will be a placeholder counting device for the raters
  
  Resp subj[nresp]; //vector declares a variable of class Resp for each 
                    //respondent (i.e. examinee) which will hold info unique
                    //to the respondent (see PCM.hpp for object detail) 

  Quest item[nquest]; //vector decalres a variable of class Quest for each
                      //item which will hold info unique to each item
                      //(see PCM.hpp for object detail) 
  

  for(int i=0;i<nresp;i++){
    subj[i].SetPar(nquest,nrater);  
    //SetPar in object Quest establishes
    //an int vector "raters" of length nrater 
    //an int vector "lat_resp" of lenghth nquest
    //a matrix "score" of dim nquest x nrater
    //"nquest" (same as above)
    //"nrater" (same as above)

    for(int j=0;j<nrater;j++){//here vec becomes a vector of integers 
      vec[j]=j;               //from 0 to nrater-1
    }

    //set up the scores for each Resp.
    for(int j=0;j<nquest;j++){       
      for(int r=0;r<nrater;r++){
	iFile>>tmp;               //read in rater r's score of Resp i's 
	                          //response to item j
	subj[i].SetScore(j,r,tmp);//and assign it to Resp's score matrix in    
      }                           //cell (j,r)    
    }
    subj[i].SetRater(nrater,vec); //assigns Resp.raters[] as a vector
  }                               //of integers from 0 to nrater-1
  
  //Initial Values-----------------------------------------------------
  
  
  //mu and tau are the examinee population mean and variance for ability
  //and will hold the current iteration
  double mu=0,tau=5,shift[nrater],scale[nrater];
  //shift and scale will the current rater severity (shift) and 
  //consistency (scale) for the current iteration

  double theta[nresp];//will hold current respondent ability iteration
  
  int task[nresp],taskc[nresp],num[nrater];
  //when updating the item parameters (w/ sum-to-0 constraint) task and taskc 
  //are used as temporary holders of the ideal scores of each Resp for the 
  //current item being updated (task) and the final constrained item (taskc)
  
  Mat<double> trans[nrater]; //will hold unnormalized transition probabilities
  //i.e. P[rater r rates (i,j) as category k | ideal score l ] is 
  //proportional to trans[r].GetElem(l,k)
  //NOTE: These MUST be scaled to add to one to be real probabilities
 
  Mat<int> xi; //matrix which will hold curent iteration's ideal scores (xi_ij)
  
  xi.Mat2(nresp,nquest); //establish dimensions of ideal score matrix as 
                         //nresp x nquest
  
  for(int r=0;r<nrater;r++){
    num[r]=0;//makes num[] a vector of all zeros (initializing count)
    trans[r].Mat2(maxcat,maxcat);//establish the dim of the rater prob. matrix
    //as maxcat x maxcat.  For items where the number of categories is fewer 
    //than maxcat, this works fine when the scaling is done over the actual 
    //set of categories for that item.
  
    shift[r]=0; //initialize rater shift starting value to 0
    scale[r]=1; //initialize rater scale starting value to 1

    calc_trans(maxcat,shift[r],scale[r],trans[r]); //calculate the rater
                                   //transition prob. matrix given the current
                                   //shift & scale and store in trans[r]
  }
  
  
  for(int i=0;i<nquest;i++){ //for each item
    item[i].SetNcat(bncat[i]);//set the number of item categories
    item[i].SetBeta(0);//initialize item difficulty starting value to 0
  }
  
  for(int i=0;i<nresp;i++){ //for each Resp 
    for(int j=0;j<subj[i].nraters;j++){
      num[subj[i].raters[j]]++ ; //sets num[r] to nresp
    }
    theta[i]=drnnof();        //Random Initial Values for ability params
  }
  
  //--------------------------------------------------------------------

  
  Mat<int> sc_r[nrater]; //will be matrix of scores by rater r
  
  
  for(int r=0;r<nrater;r++){ //for each rater
    sc_r[r].Mat2(num[r],nquest); //sets dim of sc_r to nresp x nquest
  }


  for(int i=0;i<nresp;i++){ //for each Resp 
    for(int j=0;j<nquest;j++){ //for each question
      for(int r=0;r<nrater;r++){//for each rater
	sc_r[r].SetElem(i,j,subj[i].score.GetElem(j,r));
      }//This makes sc_r[r] a matrix of r's score of i's response to
    }//item j, where sc_r[r](i,j) is bncat[j] if (i,j) not scored by r
  }

  //_________________________________________________________
  //Start Simulations
  //__________________________________________________________
  for(int isim=0;isim<nsim;isim++){ //for each simulation

    if ( (isim % 10) == 9){oLog << isim+1 << endl;}//every 10th iteration
    //output current iteration to the log.out file

    for(int i=0;i<nresp;i++){//for each respondent
      for(int j=0;j<nquest;j++){//for each item 
	subj[i].score.GetRow(j,vec);//get the scores each rater gave to 
	                            //Resp i for item j and put them in vec
	//NOTE:  when rater r does not rate (i,j) the program expects a score 
	//equal to the number of rating categories for j.  For example, if 
	//item j has 3 scoring categories, each cell of row 3 of 
	//Resp.score would contain one of {0,1,2,3}, where a 0, 1 or 2 are 
	//actual scores and a 3 indicates that the rater did not rate (i,j)    

	update_lat_resp(item[j],theta[i],trans,vec,
			subj[i].nraters,subj[i].lat_resp[j]);
	//updates the ideal score (xi_ij) and assigns it to Resp i's
	//ideal score vector (subj[i].lat_resp[j])

	xi.SetElem(i,j,subj[i].lat_resp[j]);//copy the updated ideal score 
	//to the matrix of all ideal scores

      }
      update_theta(nquest,item,theta[i],mu,tau,subj[i].lat_resp);
      //updates the current ability parameter theta_i

    }
    for(int j=0;j<nquest;j++){
      xi.GetColumn(j,task); //assign ideal scores of current item to task
      xi.GetColumn(nquest-1,taskc); //assign ideal scores of constrained 
                                    //item to taskc
      if(j != nquest-1) //if not on the final item...
	update_beta_cons(nresp,item[j],item[nquest-1] //update current item
			 ,theta,task,taskc);   //parameter and adjust 
                                               //constrained item parameter
      
      update_gamma(nresp,item[j],theta,task);//update current item's
                                             //item-step params

      for(int k=0;k<item[j].ncat-1;k++){ //output current item's
	oGamm<<item[j].gamma[k]<<"\t";   //item-step params to gamma.out
      }
      oGamm<<endl;
      oBeta<<item[j].beta<<"\t";//output current item param to beta.out
    }
    oBeta<<endl;

    update_var(nresp,theta,mu,tau); //update population ability mean and var
    oMuSi<<tau<<" "<<mu<<" ";       //and output to mu_sig.out 

    for(int r=0;r<nrater;r++){//for each rater
      update_rater(item,nresp,nquest,sc_r[r],xi, 
		   shift[r],scale[r],trans[r],.05,.1);  
      //updates current rater's shift and scale parameters
      //.05 is shift jump variance
      //.1  is scale jump variance
      //change jump var's as necessary
      
      oMuSi<<shift[r]<<" "<<scale[r]<<" ";  //output current rater shift and
                                            //scale to mu_sig.out
    }//ends for r...
    oMuSi<<endl;

    oTheta<<theta[419]<<" "<<theta[394]<<" "<<theta[285]<<" "
	  <<theta[367]<<" "<<theta[51]<<endl;
    //outputs current selected ability params to theta.out
    //adjust chosen respondents as necessary.

  }//end simulations


  //write ending time
  time(&time2);
  oLog <<"Program lasted for "<<difftime(time2,time1)/60.0<<" minutes and ended at "<< ctime(&time2);
  

  return(0);
}

