//Expands original HRM code to include multiple choice items
//
// no sum-to-zero on Beta and mu fixed at 0.
//
//Revised to include rating covariates via Option 2 of Mariano (2002)

//Written by Lou Mariano (2002), using a base of hrm.cpp

//Before running this program, select which examinee's ability parameters 
//will be retained:  see line that begins "oTheta<<"
//When running, jump variances may be adjusted here for the raters, see 
//line that starts "update_rater" within main; jump vars for the others may 
//be adjusted within the update file.
// 
//g++ -O2 -o hrmoption2.exe hrmoption2.cpp mchrm.update.cpp hrm.pcm.cpp rngen.cpp
//
//
//
//
#include<math.h>
#include<stdlib.h>
#include<iostream.h>
#include<fstream.h>
#include<iomanip.h>
#include "PCMMC.hpp"
#include <time.h>

#define drnnof() rnorm();

void cumsum(int dim,double gamma[],double eta[]); //in pcmsign
double rnorm(void);
void update_gamma(int nresp,Quest &task,double theta[],int x[]); //in update
void update_var(int nresp, double theta[], double &mu,double &tau); //in update
void update_theta(int nquest,Quest task[],double &theta,
		  double mu,double tau,int resp[]); //in update
void update_lat_resp(Quest task,double theta,Mat<double> trans[],
		     int sc[],int nrate,int &xi); //in update
void calc_trans(int ncat,double mu,double sigma,
		Mat<double> &trans); //in update
void update_rater(Quest task[],int nresp,int nquest,
		  Mat<int> x,Mat<int> xis,double &shift,double &scale,
		  Mat<double> &trans,double cshvar,double cscvar);
void update_beta(int nresp,Quest &task,double theta[],int x[]);
void update_covar_var(int df, double s2sh, double s2sc, 
		      double &varsh, double &varsc);//below
double rgamma(double);


main(int argc,char *argv[]){
  if(argc < 9+atoi(argv[3])){ //checks for proper form in command line
    cout<<"ERROR:  command-line:  hrm.exe filename nresp nquest "
	<<"ncat1 ncat2 ... ncat_nquest nrater nmc drow dcol nsim"
	<<endl;
    exit(99);
  }
  int maxcat=0; //will hold the maxium number of categories among all items
  int nquest=atoi(argv[3]),nsim=atoi(argv[8+nquest]),nresp=atoi(argv[2]),
    nrater=atoi(argv[4+nquest]); //declares and assigns 
                                 //number of CR questions (nquest)
                                 //number of simulations (nsim)
                                 //number of examinees (nresp)
                                 //number of raters (nrater)

  int nmc=atoi(argv[5+nquest]); //number of Multiple Choice questions 
  int drow=atoi(argv[6+nquest]); //number of rows in design matrix
  //also number of unique rater x covariate combinations in data
  int dcol=atoi(argv[7+nquest]); //number of columns in design matrix
  //also sum number of rater and covariate values (after constraints)

  int totquest;
  totquest = nquest + nmc;

  int bncat[nquest]; //will hold number of categories for each question
  for(int i=0;i<nquest;i++){
    bncat[i]=atoi(argv[4+i]); //assign number of categories
    maxcat = bncat[i] > maxcat ? bncat[i] : maxcat;
  } //calculate and assign maxium number of categories

  char *infile= argv[1]; //assign file containing the data

  //ESTABLISH TIME...Start and End time and total minutes of run time
  //                 be written to the log.out file
  time_t time1, time2;  //declaration of time variables
  time(&time1);
  
  //confirm command line input back to the screen
  cout<<"Data in "<<argv[1]<<endl;
  cout<<nresp<<" Responses on"<<endl;
  cout<<nmc<<" Multiple Choice and"<<endl; 
  cout<<nquest<<" CR Questions by"<<endl;
  cout<<nrater<<" Raters scored"<<endl;
  for(int i=0;i<nquest;i++)
    cout<<"0-"<<bncat[i]-1<<endl;
  cout<<"design matrix:  "<<drow<<" rows x "<<dcol<<" columns"<<endl;
  cout<<nsim<<" Simulations"<<endl;

  //declare output files
  char *outbeta="linout/beta.out"; 
  //will hold item parameter iterations (in columns)
  
  char *outgamma="linout/gamma.out";
  //will hold item-step parameter iterations
  //one row per item-step per iteration

  char *outmusig="linout/mu_sig.out";
  //will hold population and rater params
  //ea. row:  var mean shift(1) scale(1) ... shift(drow) scale(drow)

  char *outtheta="linout/theta.out";
  //will hold selected ability params (in columns)

  char *outlog="linout/log.out";
  //will hold log file tracking time, command line info, and iterations (by 10)
  char *outshift="linout/eta.out";//bias covariate params.
  char *outscale="linout/tau.out";//variability covar params

  //continued file set-up
  ofstream oLog(outlog,ios::out);
  ofstream oSH(outshift,ios::out);
  ofstream oSC(outscale,ios::out);
  ofstream oTheta(outtheta,ios::out);
  ofstream oGamm(outgamma,ios::out);
  ofstream oBeta(outbeta,ios::out);
  ofstream oMuSi(outmusig,ios::out);
  ifstream iFile(infile,ios::in); //establish input file

  //input files for design matrix
  ifstream iMC("/TOIL-U2/ltm/MAIN/GSE/Code/hold/s1MCGSE.txt");
  ifstream iX("s1designfull.txt");
  ifstream iXTX("s1dXTX.txt");
  ifstream iXTXI("s1dXTXI.txt");
  ifstream iCD("s1dCDXTXI.txt");

  fstream oInit;
  
  //ESTABLISH TIME...continued
  oLog <<setprecision(3);
  oLog <<"Program started at "<< ctime(&time1) << endl;

  //confirm command line input to the log file
  oLog<<"Data in "<<argv[1]<<endl;
  oLog<<nresp<<" Responses on"<<endl;
  oLog<<nmc<<" Multiple Choice and"<<endl; //*****
  oLog<<nquest<<" CR Questions by"<<endl;
  oLog<<nrater<<" Raters scored"<<endl;
  for(int i=0;i<nquest;i++)
    oLog<<"0-"<<bncat[i]-1<<endl;
  oLog<<"design matrix:  "<<drow<<" rows x "<<dcol<<" columns"<<endl;
  oLog<<nsim<<" Simulations"<<endl;

  int tmp; //will be used as a temporary holder for input from the datafile

  int vec[drow];//will be a placeholder counting device for the raters
  
  Resp subj[nresp]; //vector declares a variable of class Resp for each 
                    //respondent (i.e. examinee) which will hold info unique
                    //to the respondent (see PCM.hpp for object detail) 

  Quest item[totquest]; //vector decalres a variable of class Quest for each
                      //item which will hold info unique to each item
                      //(see PCM.hpp for object detail) 
  

  for(int i=0;i<nresp;i++){
    subj[i].SetPar(nquest,drow,nmc);  
    //SetPar in object Resp establishes
    //an int vector "raters" of length drow 
    //an int vector "lat_resp" of lenghth nquest+nmc
    //a matrix "score" of dim nquest x drow
    //"nquest" (same as above)
    //"drow" (same as above)
    //"nmc" (same as above)

    for(int j=0;j<drow;j++){//here vec becomes a vector of integers 
      vec[j]=j;               //from 0 to drow-1
    }

    //set up the scores for each Resp.
    for(int j=0;j<nquest;j++){       
      for(int r=0;r<drow;r++){
	iFile>>tmp;               //read in rater r's score of Resp i's 
	                          //response to item j
	subj[i].SetScore(j,r,tmp);//and assign it to Resp's score matrix in    
      }                           //cell (j,r)    
    }
    subj[i].SetRater(drow,vec); //assigns Resp.raters[] as a vector
  }                               //of integers from 0 to drow-1
  
  //Set up design matrix______________________________________________

  double dtmp;
  double X[drow][dcol], XT[dcol][drow], XTX[dcol][dcol], XTXI[dcol][dcol];
  //X is the design matrix
  //XT is the transpose of X
  //XTX is XT * X
  //XTXI is the inverse of XTX
  for (int g=0; g<drow; g++){
    for (int h=0; h<dcol; h++){
      iX >> dtmp;
      X[g][h]=dtmp;
      XT[h][g]=dtmp;
    }
  }

  for (int g=0; g<dcol; g++){
    for (int h=0; h<dcol; h++){
      iXTX >> dtmp;
      XTX[g][h]=dtmp;
      iXTXI >> dtmp;
      XTXI[g][h]=dtmp;
    }
  }

  double XTXIXT[dcol][drow];
  //XTXIXT is XTXI * XT
  for (int g=0; g<dcol; g++){
    for (int h=0; h<drow; h++){
      XTXIXT[g][h]=0;
    }
  }
  for (int g=0; g<dcol; g++){
    for (int h=0; h<drow; h++){
      for (int m=0; m<dcol; m++){
	XTXIXT[g][h]+=XTXI[g][m]*XT[m][h];
      }
    }
  }

  double V[dcol][dcol];
  //holds Cholesky Decomposition for MVN draw
  //in update of rater effects and covars
  for (int g=0; g<dcol; g++){
    for (int h=0; h<dcol; h++){
      iCD >> dtmp;
      V[g][h]=dtmp;
    }
  }





  //Initial Values-----------------------------------------------------
  
  
  //mu and tau are the examinee population mean and variance for ability
  //and will hold the current iteration
  double mu=0.0,tau=0.444,shift[drow],scale[drow];
  //shift and scale will the current rater severity (shift) and 
  //consistency (scale) for the current iteration

  double theta[nresp];//will hold current respondent ability iteration
  
  int tsk[nresp],tskc[nresp],num[drow];
  //when updating the item parameters (w/ sum-to-0 constraint) tsk and tskc 
  //are used as temporary holders of the ideal scores of each Resp for the 
  //current item being updated (tsk) and the final constrained item (tskc)
  
  Mat<double> trans[drow]; //will hold unnormalized transition probabilities
  //i.e. P[rater r rates (i,j) as category k | ideal score l ] is 
  //proportional to trans[r].GetElem(l,k)
  //NOTE: These MUST be scaled to add to one to be real probabilities
 
  Mat<int> xi; //matrix which will hold curent iteration's ideal scores (xi_ij)
  
  xi.Mat2(nresp,totquest); //establish dimentions of ideal score matrix as 
                         //nresp x (nquest+nmc)
  
  int mctmp;
  for (int i=0; i<nresp; i++){//cycle through the examinees 
    iMC >> mctmp; //discard MC ID score
    for (int y=nquest; y<totquest; y++){//cycle through the MC scores 
      iMC >> mctmp; //read in MC score
      subj[i].lat_resp[y]=mctmp; //assign MC score to subject  
      xi.SetElem(i,y,mctmp);//assign MC score to ideal score matrix 
    }
  }


  for(int r=0;r<drow;r++){
    num[r]=0;//makes num[] a vector of all zeros (initializing count)
    trans[r].Mat2(maxcat,maxcat);//establish the dim of the rater prob. matrix
    //as maxcat x maxcat.  For items where the number of categories is fewer 
    //than maxcat, this works fine when the scaling is done over the actual 
    //set of categories for that item.
  
    shift[r]=0; //initialize rater shift starting value to 0
    scale[r]=1; //initialize rater scale starting value to 1

    calc_trans(maxcat,shift[r],scale[r],trans[r]); //calculate the rater
                                   //transition prob. matrix given the current
                                   //shift & scale and store in trans[r]
  }
  
  
  for(int i=0;i<nquest;i++){ //for each item
    item[i].SetNcat(bncat[i]);//set the number of item categories
    item[i].SetBeta(0);//initialize item difficulty starting value to 0
  }
  for(int i=0;i<nmc;i++){ //also for MC items
    item[i+nquest].SetNcat(2);//set the number of item categories
    item[i+nquest].SetBeta(0);//initialize item difficulty starting value to 0
  }
  

  for(int j=0;j<totquest;j++){
    for(int k=0; k<item[j].ncat-1;k++){
      item[j].gamma[k]=0;
    }
  }

  for(int i=0;i<nresp;i++){ //for each Resp 
    for(int j=0;j<subj[i].nraters;j++){
      num[subj[i].raters[j]]++ ; //sets num[r] to nresp
    }
    theta[i]=drnnof();        //Random Initial Values for ability params
  }
  
  double eta[dcol], kappa[dcol], varsh=1, varsc=1;
  //eta is vector of rater and covariate shift effects
  //kappa is vector of rater and covariate scale effects
  //sig2sh is var on shift effects
  //sig2sc is var on scale effects
  for (int h=0; h<dcol; h++){
    eta[h]=0;
    kappa[h]=0;
  }
  double etahat[dcol], kappahat[dcol];
  for (int g=0; g<dcol; g++){
    etahat[g]=0;
    kappahat[g]=0;
  }

  double etadraw[dcol], kappadraw[dcol];
  double etad[dcol], kappad[dcol];
  //used in update of eta and kappa


  //  double Xeta[drow], Xkappa[drow]; 
  double Xetahat[drow], Xkappahat[drow];
  //holds current design matrix x rater/covar effects
  //"hat"'s used in update of rater/covar effects and var
  //non-"hat"'s used in update_rater
  //  double meansh, meansc; //used in update_rater
  int df=drow-dcol;//used in update of rater/covar effects and var

  double s2sh, s2sc;//used in update rater/covar effects and var

  //--------------------------------------------------------------------

  
  Mat<int> sc_r[drow]; //will be matrix of scores by rater r
  
  
  for(int r=0;r<drow;r++){ 
    sc_r[r].Mat2(num[r],nquest); 
  }

  for(int i=0;i<nresp;i++){ //for each Resp 
    for(int j=0;j<nquest;j++){ //for each question
      for(int r=0;r<drow;r++){//for each rater
	sc_r[r].SetElem(i,j,subj[i].score.GetElem(j,r));
      }//This makes sc_r[r] a matrix of r's score of i's response to
    }//item j, where sc_r[r](i,j) is bncat[j] if (i,j) not scored by r
  }



  double betasd[totquest];
  for (int j=0;j<totquest-1;j++){
    betasd[j]=.15;
  } 


  //_________________________________________________________
  //Start Simulations
  //__________________________________________________________


  for(int isim=0;isim<nsim;isim++){ //for each simulation

    if ( (isim % 10) == 9){oLog << isim+1 << endl;}//every 10th iteration
    //output current iteration to the log.out file

    for(int i=0;i<nresp;i++){//for each respondent
      for(int j=0;j<nquest;j++){//for each CR item 
	subj[i].score.GetRow(j,vec);//get the scores each rater gave to 
	                            //Resp i for item j and put them in vec
	//NOTE:  when rater r does not rate (i,j) the program expects a score 
	//equal to the number of rating categories for j.  For example, if 
	//item j has 3 scoring categories, each cell of row 3 of 
	//Resp.score would contain one of {0,1,2,3}, where a 0, 1 or 2 are 
	//actual scores and a 3 indicates that the rater did not rate (i,j)    

	update_lat_resp(item[j],theta[i],trans,vec,
			subj[i].nraters,subj[i].lat_resp[j]);
	//updates the ideal score (xi_ij) and assigns it to Resp i's
	//ideal score vector (subj[i].lat_resp[j])

	xi.SetElem(i,j,subj[i].lat_resp[j]);//copy the updated ideal score 
	//to the matrix of all ideal scores

      }
      update_theta(totquest,item,theta[i],mu,tau,subj[i].lat_resp);
      //updates the current ability parameter theta_i

    }

    for(int j=0;j<totquest;j++){

      xi.GetColumn(j,tsk); //assign ideal scores of current item to tsk

      update_beta(nresp,item[j],theta,tsk); //update current item
                                     
      update_gamma(nresp,item[j],theta,tsk);//update current item's
                                             //item-step params

      for(int k=0;k<item[j].ncat-1;k++){ //output current item's
	oGamm<<item[j].gamma[k]<<"\t";   //item-step params to gamma.out
      }
      oGamm<<endl;
      oBeta<<item[j].beta<<"\t";//output current item param to beta.out
    }
    oBeta<<endl;

    update_var(nresp,theta,mu,tau); //update population ability mean and var
    oMuSi<<tau<<" "<<mu<<" ";       //and output to mu_sig.out 

    if(nquest > 0){

      for(int r=0;r<drow;r++){//for each rater
	update_rater(item,nresp,nquest,sc_r[r],xi, 
		     shift[r],scale[r],trans[r],.05,.1);  
	//updates current rater's shift and scale parameters
	//.05 is shift jump variance
	//.1  is scale jump variance
	//change jump var's as necessary
	
	oMuSi<<shift[r]<<" "<<scale[r]<<" ";  //output current rater shift and
	                                      //scale to mu_sig.out

      }//ends for r... 


      //set up for draws of varsh, varsc and draws of 
      //eta|varsh, kappa|varsc

      for (int g=0; g<dcol; g++){
	etahat[g]=0;//prepare for update rater and covar effects
	kappahat[g]=0;
	for (int h=0; h<drow; h++){
	  etahat[g]  += XTXIXT[g][h] * shift[h];
	  kappahat[g]+= XTXIXT[g][h] * scale[h];
	}
      }


      for (int g=0; g<drow; g++){
	Xetahat[g]=0;
	Xkappahat[g]=0;
	for (int h=0; h<dcol; h++){
	  Xetahat[g]+=X[g][h]*etahat[h];
	  Xkappahat[g]+=X[g][h]*kappahat[h];
	}
      }

      s2sh=0;
      s2sc=0;
      for (int g=0; g<drow; g++){
	s2sh+= pow( (shift[g]-Xetahat[g]) ,2);
	s2sc+= pow( (scale[g]-Xkappahat[g]) ,2);
      }
      s2sh=s2sh/df;
      s2sc=s2sc/df;

      update_covar_var(df, s2sh, s2sc, varsh, varsc);

      for (int g=0; g<dcol; g++){//draw from MVN(0,I)
	etadraw[g]=drnnof();  
	kappadraw[g]=drnnof();
      }
      for (int g=0; g<dcol; g++){//convert to MVN(0,XTXI)
	etad[g]=0;
	kappad[g]=0;
	for (int h=0; h<dcol; h++){
	  etad[g]  += V[g][h] * etadraw[h];
	  kappad[g]+= V[g][h] * kappadraw[h];
	}
      }
      for (int g=0; g<dcol; g++){//convert to MVN(etahat,XTXI*varsh)
	eta[g]= (varsh * etad[g]) + etahat[g];
	oSH << eta[g] << " ";
	kappa[g]= (varsc * kappad[g]) + kappahat[g];
	oSC << kappa[g] << " ";
      }//and convert to MVN(kappahat,XTXI*varsc)
      //end update eta,kappa
      oSH << varsh<<endl;
      oSC << varsc<<endl;

    }//ends if
    oMuSi<<endl;

    oTheta<<theta[0]<<" "<<theta[1]<<" "<<theta[13]<<" "
	  <<theta[3796]<<" "<<theta[128]<<endl;



  }//end simulations


  //write ending time
  time(&time2);
  oLog <<"Program lasted for "<<difftime(time2,time1)/60.0<<" minutes and ended at "<< ctime(&time2);
  


  return(0);
}


//##############################################################3

void update_covar_var(int df, double s2sh, double s2sc, 
		      double &varsh, double &varsc){
  double chish,chisc;
  double gal=df/2;
  //  draw 2 Gamma(gal,1) r.v.'s
  chish=rgamma(gal);
  chisc=rgamma(gal);
  //convert to Inv-Chi-sq r.v.'s
  varsh= (df * s2sh) / (2*chish);
  varsc= (df * s2sc) / (2*chisc);
  return;
}



