//This program fits rating modality as a rating covariate to the FCAT data
//using the HRM with rater and modality effect contained at the same level 
//as the data (Option 1 in Mariano 2002).

//this version restricts raters to sum-to-zero within mode 1 and across 
//modes 2&3 and updates the raters in 2 sets using Metropolis steps.  

//Written by Lou Mariano (2002), using a base of hrm.cpp

//g++ -O2 -o mode.hrm.exe mode.hrm.cpp mode.update.cpp hrm.pcm.cpp rngen.cpp
#include<math.h>
#include<stdlib.h>
#include<iostream.h>
#include<fstream.h>
#include<iomanip.h>
#include "PCM.hpp"
#include <time.h>
#include <malloc.h>

#define drnunf() drand48()
#define drnnof() rnorm();

unsigned short int *seed48(unsigned short int seed16v[3]);

void rdisc(int nr,int imin,int nmass,double *prob,int *ir){
  for(int j=0;j<nr;j++){
    ir[j]=imin;
    double r=drnunf();
    int i=0;
    while(r>prob[i]){
      ++ir[j];
      ++i;
    }
    return;
  }
}

double lognorm_dens(double x, double mean, double sd);
void pcm (Quest task,double theta,double prob[]);
void cumsum(int dim,double gamma[],double eta[]);
double rnorm(void);
void update_gamma(int nresp,Quest &task,double theta[],int x[]);
void update_var(int nresp, double theta[], double &mu,double &tau);
void update_theta(int nquest,Quest task[],double &theta,
		  double mu,double tau,int resp[]);
void calc_trans(int ncat,double mur,double sigmar,double mum,double sigmam,
		Mat<double> &trans);
void update_beta_cons(int nresp,Quest &task,Quest &taskc
		 ,double theta[],int x[],int xc[]);


main(int argc,char *argv[]){ 

  if(argc < 7+atoi(argv[3])){
    cout<<"ERROR:  command-line:  hrm.exe filename nresp nquest "
	<<"ncat1 ncat2 ... ncat_nquest nrater nmode nsim"
	<<endl;
    exit(99);
  }
  
  int nquest=atoi(argv[3]),nsim=atoi(argv[6+nquest]),nresp=atoi(argv[2]),
    nrater=atoi(argv[4+nquest]),nmode=atoi(argv[5+nquest]);
  //nquest is number of items
  //nsim is number of mcmc simulations
  //nresp is the number of examinees
  //nrater is the total number of raters

  int maxcat=0;
  int bncat[nquest];
  for(int j=0;j<nquest;j++){
    bncat[j]=atoi(argv[4+j]);
    //bncat is a vector containing the number of response 
    //categories for each item.
    maxcat = bncat[j] > maxcat ? bncat[j] : maxcat;
  }

  char *infile= argv[1];
  //infile is a pointer to the pointer to the input file containing 
  //the ratings.  

  
  //ESTABLISH TIME
  time_t time1, time2;  //declaration of time variables
  time(&time1);

  //confirms the input parameters back to the user 
  cout<<"Data in "<<argv[1]<<endl;
  cout<<nresp<<" Responses on"<<endl;
  cout<<nquest<<" Questions by"<<endl;
  cout<<nrater<<" Raters scored"<<endl;
  cout<<"in "<<nmode<<" Modes"<<endl;
  for(int i=0;i<nquest;i++)
    cout<<"0-"<<bncat[i]-1<<endl;
  cout<<nsim<<" Simulations"<<endl;

  /////////////////////////////SET SEED//////////////////////SSS
  unsigned short int theseeds[] = { 10 , 26, 91 };
  seed48(theseeds);//set the seed for drand48
  /////////////////////////////SET SEED//////////////////////SSS

  //set up input and output files
  char *outbeta="outputr/beta.out";
  char *outgamma="outputr/gamma.out";
  char *outmusigr="outputr/mu_sig_rater.out";
  char *outmusigm="outputr/mode.out";
  char *outtheta="outputr/theta.out";
  char *outlog="outputr/log.out";

  ofstream oLog(outlog,ios::out);
  ofstream oTheta(outtheta,ios::out);
  ofstream oGamm(outgamma,ios::out);
  ofstream oBeta(outbeta,ios::out);
  ofstream oMuSir(outmusigr,ios::out);
  ofstream oMode(outmusigm,ios::out);
  ifstream iFile(infile,ios::in);
  fstream oInit;
  
  //ESTABLISH TIME (cont.)
  oLog <<setprecision(3);
  oLog <<"Program started at "<< ctime(&time1) << endl;

  //confirm command line to log file
  oLog<<"Data in "<<argv[1]<<endl;
  oLog<<nresp<<" Responses on"<<endl;
  oLog<<nquest<<" Questions by"<<endl;
  oLog<<nrater<<" Raters scored"<<endl;
  oLog<<"in "<<nmode<<" Modes"<<endl;
  for(int i=0;i<nquest;i++)
    oLog<<"0-"<<bncat[i]-1<<endl;
  oLog<<nsim<<" Simulations"<<endl;


  int tmp;     //used in reading in the data
  int vec[nrater];  
  
  Resp subj[nresp];     //set up a vector of subjects of class Resp
  Quest item[nquest];   //se up a vector of items of class Quest

  int ratmode[nresp][nquest][nrater];  //set up an array which holds the 
                                       //modality under which rater r 
                                       //rated item j for examinee i
                                       //0 = not rated
  
  for(int i=0;i<nresp;i++){
    subj[i].SetPar(nquest,nrater);  //for subject i, set up a 
                                    //question x rater matrix 
    for(int j=0;j<nrater;j++){
      vec[j]=j;
    }
    for(int k=0;k<nquest;k++){
      for(int j=0;j<nrater;j++){
	iFile>>tmp;                   
	subj[i].SetScore(k,j,tmp);    
      }                              
    }
    subj[i].SetRater(nrater,vec);
  }

  for(int i=0;i<nresp;i++){  //fill in rater modality array  
    for(int j=0;j<nquest;j++){ 
      for(int r=0;r<nrater;r++){  
	iFile>>tmp; 
	ratmode[i][j][r] = tmp;  
      }  
    }  
  }  //end fill in rater modality array

  //create 2 smaller modality arrays 
  int irmode[nquest][nrater];//[ item x rater] use in update xi
  int rimode[nrater][nquest];//[rater x item]  use in update raters

  int d; // counter for while loop  

  for(int j=0;j<nquest;j++){  
    for(int r=0;r<nrater;r++){  
      d = 0; 
      while ( ratmode[d][j][r]==0 && d<nresp-1 ) {   
	d++;  
      } 
      irmode[j][r] = ratmode[d][j][r];  
      rimode[r][j] = ratmode[d][j][r];  
    }  
  } 

  for(int r=0;r<nrater;r++){
    for(int j=0;j<nquest;j++){
    }
  }  
  //END create smaller modality arrays 


  
  //Initial Values-----------------------------------------------------
  
  nmode ++;//make extra as tracking device for not.rated=0

  double mu=0,tau=5,shiftr[nrater],scaler[nrater],shiftm[nmode],scalem[nmode];
  double theta[nresp];
  
  int task[nresp],taskc[nresp],num[nrater];
  
  Mat<double> trans[nrater][nmode];
  
  Mat<int> xi; //matrix for ideal scores
  
  xi.Mat2(nresp,nquest); //set up matrix of ideals to be of 
                         // dim #subjects x #items
  
  for(int m=0; m<nmode; m++){//initialize mode shift and scale
    shiftm[m]=0;
    scalem[m]=1;
  }

  for(int i=0;i<nrater;i++){//initialize rater shift and scale
    num[i]=0;
    shiftr[i]=0;
    scaler[i]=1;
    for(int m=0; m<nmode; m++){
      trans[i][m].Mat2(maxcat,maxcat);
      calc_trans(maxcat,shiftr[i],scaler[i],shiftm[m],scalem[m],trans[i][m]);
    }
  }//and set up normalizing constants for updating latent and rater
 
  
  for(int i=0;i<nquest;i++){
    item[i].SetNcat(bncat[i]);//sets number of categories for ea. item
    item[i].SetBeta(0);//sets ea. item param to 0
  }
  
  for(int i=0;i<nresp;i++){
    for(int j=0;j<subj[i].nraters;j++){
      num[subj[i].raters[j]]++ ;
    }
    theta[i]=drnnof();        //Random Initial Values for Theta
  }
  
  //--------------------------------------------------------------------

  
  Mat<int> sc_r[nrater];
  Mat<int> xi_r[nrater];
  int r,l;
  
  
  for(int j=0;j<nrater;j++){
    xi_r[j].Mat2(num[j],nquest);
    sc_r[j].Mat2(num[j],nquest);
  }
  for(int i=0;i<nresp;i++){ //for each Resp 
    for(int j=0;j<nquest;j++){ //for each question
      for(int r=0;r<nrater;r++){//for each rater
	sc_r[r].SetElem(i,j,subj[i].score.GetElem(j,r));
      }//This makes sc_r[r] a matrix of r's score of i's response to
    }//item j, where sc_r[r](i,j) is bncat[j] if (i,j) not scored by r
  }


    //Vars used for update latent_var, rater, mode
  double prob[maxcat],cprob[maxcat+1],rr[maxcat];
  double sum=0,z;
  double phi,cphi,norm[nrater][maxcat];
  int nr=1,imin=0;
  int ir[nr];


  const int nm1r=7;//number of modality 1 raters
  const int nm23r=31;//number of modality 2&3 raters
  const int last1=6;//vector index of rater 7 (last m_1 rater)
  const int last=37;//vector index of final rater (38)

  double cshiftr1[nm1r], cshiftr23[nrater], cscaler1[nm1r], cscaler23[nrater], cshiftm, cscalem, llike_diff=0;
  double cshiftl,cscalel;
  Mat<double> ctrans1[nm1r][nmode];//used in update rater and lat_resp
  for(int r=0;r<nm1r;r++){ 
    for (int m=0; m<nmode; m++){
    ctrans1[r][m].Mat2(maxcat,maxcat);
    }
  }
  Mat<double> ctrans23[nrater][nmode];//used in update rater and lat_resp
  for(int r=nm1r;r<nrater;r++){
    for (int m=0; m<nmode; m++){
      ctrans23[r][m].Mat2(maxcat,maxcat);
    }
  }
  Mat<double> ctransl[nmode];//used in update rater and lat_resp
  for (int m=0; m<nmode; m++){
    ctransl[m].Mat2(maxcat,maxcat);
  }
  Mat<double> ctran;//used in update mode
  ctran.Mat2(maxcat,maxcat);
  int latresp, thescore;
  double ratnorm[maxcat],cratnorm[maxcat],hold[maxcat],holdc[maxcat];
  double cshrvar = .05, cscrvar = .1; //rater param jump var (actually sd)
  double shjmpvar = .025, scjmpvar = .025;//mode param jump var (actually sd)
  

  double shsd[nrater], scsd[nrater];//holds shift and scale jump dist s.d.
  for (int r=0; r<nrater; r++){
    shsd[r]=.01;
    scsd[r]=.01;
  }
  for (int r=0; r<nm1r; r++){
    shsd[r]=.0125;
    scsd[r]=.01;
  }

  double mshsd[nmode], mscsd[nmode];//holds mode shift and scale jump dist s.d.
  for (int m=0; m<nmode; m++){
    mshsd[m]=.04;
    mscsd[m]=.04;
  }

  double prevpsi[nrater],prevphi[nrater];//track scale jump dist
  int psijump[nrater],phijump[nrater];
  for (int r=0; r<nrater; r++){
    prevphi[r] = 0;
    prevpsi[r] = 1;
    phijump[r] = 0;
    psijump[r] = 0;
  }

  double prevmpsi[nmode];//track mode scale jump dist
  int mpsijump[nmode];
  for (int m=0; m<nmode; m++){
    prevmpsi[r] = 1;
    mpsijump[r] = 0;
  }

  double prevmphi[nmode];//track mode shift jump dist
  int mphijump[nmode];
  for (int m=0; m<nmode; m++){
    prevmphi[r] = 0;
    mphijump[r] = 0;
  }


  //_________________________________________________________
  //Start Simulations
  //__________________________________________________________
  for(int isim=0;isim<nsim;isim++){
    if ( (isim % 10) == 9){oLog << isim+1 << endl;}
    for(int i=0;i<nresp;i++){ 

      for(int j=0;j<nquest;j++){


	//UPDATING IDEAL SCORES

	subj[i].score.GetRow(j,vec); //get scores for subject i item j

	//begin update XI *************************************************

	sum=0;

	pcm(item[j],theta[i],prob);
  
	for(int r=0;r<subj[i].nraters;r++){          //RATERS FOLLOW r
	  for(int k=0;k<item[j].ncat;k++){           //CATEGORIES FOLLOW k
	    norm[r][k]=0;
	  }}
	for(int r=0;r<subj[i].nraters;r++){          //RATERS FOLLOW r 
	  for(int k=0;k<item[j].ncat;k++){           //CATEGORIES FOLLOW k
	    for(int l=0;l<item[j].ncat;l++){         //REPEAT CATEGORIES FOLLOW l
	      norm[r][k] += trans[r][irmode[j][r]].GetElem(k,l);
	    }
	  }
	}
  

	for(int k=0;k<item[j].ncat;k++){
	  rr[k]=1;
	  for(int r=0;r<subj[i].nraters;r++){
	    phi= vec[r]==item[j].ncat ? 1 : 
	      trans[r][irmode[j][r]].GetElem(k,vec[r])/norm[r][k];
	    rr[k] *=phi;
	  }
	  rr[k] *= prob[k];
	  sum += rr[k];
	}
	cprob[item[j].ncat]=0;
	cprob[item[j].ncat-1]=1;
	cprob[0]=rr[0]/sum;
	for(int k=1;k< item[j].ncat -1;k++){
	  cprob[k]=cprob[k-1]+rr[k]/sum;
	}
	rdisc(nr,imin,item[j].ncat,cprob,ir);
	subj[i].lat_resp[j]= ir[0];

	//end update XI ***************************************************
 
	xi.SetElem(i,j,subj[i].lat_resp[j]);
	//updates current ideal score matrix 

      }
      //END UPDATING IDEAL SCORES



      //update ability params
      update_theta(nquest,item,theta[i],mu,tau,subj[i].lat_resp);
    }

    for(int j=0;j<nquest;j++){
      xi.GetColumn(j,task);
      xi.GetColumn(nquest-1,taskc);
      if(j != nquest-1)
	update_beta_cons(nresp,item[j],item[nquest-1]
			 ,theta,task,taskc);   // Updating Beta
      
      update_gamma(nresp,item[j],theta,task);  // Updating Gamma
      for(int k=0;k<item[j].ncat-1;k++){
	oGamm<<item[j].gamma[k]<<"\t";
      }
      oGamm<<endl;
      oBeta<<item[j].beta<<"\t";
    }
    oBeta<<endl;
    update_var(nresp,theta,mu,tau);
    oMuSir<<tau<<" "<<mu<<" ";


    //BEGIN UPDATING RATERS

    //START MODALITY 1 RATERS

    //starting rater shift


    llike_diff=0;
    cshiftr1[last1]=shiftr[last1];

    for(int r=0;r<nm1r-1;r++){  

      z=drnnof();
      cshiftr1[r]=shiftr[r]+shsd[r]*z;
      cshiftr1[last1]-=shsd[r]*z;
  
      for(int m=0; m<nmode; m++){
	calc_trans(maxcat,cshiftr1[r],scaler[r],shiftm[m],scalem[m],ctrans1[r][m]);
      }
    }
    for(int m=0; m<nmode; m++){
      calc_trans(maxcat,cshiftr1[last1],scaler[last1],shiftm[m],scalem[m],ctrans1[last1][m]);
    }

  
    for(int r=0;r<nm1r;r++){
      for(int j=0;j<nquest;j++){
	for(int i=0;i<nresp;i++){
	  thescore=sc_r[r].GetElem(i,j);
	  if(thescore<item[j].ncat){
	    latresp=xi.GetElem(i,j);
	    trans[r][rimode[r][j]].GetRow(latresp,hold);
	    ctrans1[r][rimode[r][j]].GetRow(latresp,holdc);
	    cumsum(item[j].ncat,hold,ratnorm);
	    cumsum(item[j].ncat,holdc,cratnorm);
	    phi = log(hold[thescore]/ratnorm[item[j].ncat-1]);
	    cphi= log(holdc[thescore]/cratnorm[item[j].ncat-1]);
	    llike_diff += cphi-phi;
	  }
	}
      }
    }

    for(int r=0;r<nm1r-1;r++){ 
      llike_diff += .5*(pow(shiftr[r],2)/1.0)-.5*(pow(cshiftr1[r],2)/10.0);
    }

    if(drnunf()<exp(llike_diff)){
      for(int r=0;r<nm1r;r++){ 
	shiftr[r]=cshiftr1[r];
	for(int m=1;m<nmode;m++){
	  for(int k=0;k<maxcat;k++){
	    for(int l=0;l<maxcat;l++){
	      trans[r][m].SetElem(k,l,ctrans1[r][m].GetElem(k,l));
	    }
	  }
	}
      }
    }

    //starting rater scale

    llike_diff=0;
    cscaler1[last1]=scaler[last1];

    for(int r=0;r<nm1r-1;r++){  

      z=drnnof();
      cscaler1[r]=scaler[r]*exp(scsd[r]*z);
      cscaler1[last1]=cscaler1[last1]/exp(scsd[r]*z);

      for(int m=0; m<nmode; m++){
	calc_trans(maxcat,shiftr[r],cscaler1[r],shiftm[m],scalem[m],ctrans1[r][m]);
      }  
    }
    for(int m=0; m<nmode; m++){
      calc_trans(maxcat,shiftr[last1],cscaler1[last1],shiftm[m],scalem[m],ctrans1[last1][m]);
    }  

    for(int r=0;r<nm1r;r++){
      for(int j=0;j<nquest;j++){
	for(int i=0;i<nresp;i++){
	  thescore=sc_r[r].GetElem(i,j);
	  if(thescore<item[j].ncat){
	    latresp=xi.GetElem(i,j);
	    trans[r][rimode[r][j]].GetRow(latresp,hold);
	    ctrans1[r][rimode[r][j]].GetRow(latresp,holdc);
	    cumsum(item[j].ncat,hold,ratnorm);
	    cumsum(item[j].ncat,holdc,cratnorm);
	    phi = log(hold[thescore]/ratnorm[item[j].ncat-1]);
	    cphi= log(holdc[thescore]/cratnorm[item[j].ncat-1]);
	    llike_diff += cphi-phi;
	  }
	}
      }
    }


    for(int r=0;r<nm1r-1;r++){ 
      llike_diff += .5*(pow(log(scaler[r]),2)/10.0)-.5*(pow(log(cscaler1[r]),2)/10.0) - log(cscaler1[r]) + log(scaler[r]);
      llike_diff += log( lognorm_dens(scaler[r],log(cscaler1[r]),cscrvar) ) - log( lognorm_dens(cscaler1[r],log(scaler[r]),cscrvar) );
    }


    if(drnunf()<exp(llike_diff)){
      for(int r=0;r<nm1r;r++){ 
	scaler[r]=cscaler1[r];
	for(int m=1;m<nmode;m++){
	  for(int k=0;k<maxcat;k++){
	    for(int l=0;l<maxcat;l++){
	      trans[r][m].SetElem(k,l,ctrans1[r][m].GetElem(k,l));
	    }
	  }
	}
      }
    }

    for(int r=0;r<nm1r;r++){ 
      oMuSir<<shiftr[r]<<" "<<scaler[r]<<" ";             

      if (scaler[r] != prevpsi[r]){
	psijump[r]++;
	prevpsi[r] = scaler[r];
      }
      if (shiftr[r] != prevphi[r]){ 
	phijump[r]++; 
	prevphi[r] = shiftr[r]; 
      } 
    }

  //UPDATE MODALITY 2 & 3 RATERS

    //starting rater shift

    llike_diff=0;
    cshiftr23[last]=shiftr[last];

    for(int r=nm1r;r<nrater-1;r++){  

      z=drnnof();
      cshiftr23[r]=shiftr[r]+shsd[r]*z;
      cshiftr23[last]-=shsd[r]*z;
  
      for(int m=0; m<nmode; m++){
	calc_trans(maxcat,cshiftr23[r],scaler[r],shiftm[m],scalem[m],ctrans23[r][m]);
      }
    }
    for(int m=0; m<nmode; m++){
      calc_trans(maxcat,cshiftr23[last],scaler[last],shiftm[m],scalem[m],ctrans23[last][m]);
    }

  
    for(int r=nm1r;r<nrater;r++){    
      for(int j=0;j<nquest;j++){
	for(int i=0;i<nresp;i++){
	  thescore=sc_r[r].GetElem(i,j);
	  if(thescore<item[j].ncat){
	    latresp=xi.GetElem(i,j);
	    trans[r][rimode[r][j]].GetRow(latresp,hold);
	    ctrans23[r][rimode[r][j]].GetRow(latresp,holdc);
	    cumsum(item[j].ncat,hold,ratnorm);
	    cumsum(item[j].ncat,holdc,cratnorm);
	    phi = log(hold[thescore]/ratnorm[item[j].ncat-1]);
	    cphi= log(holdc[thescore]/cratnorm[item[j].ncat-1]);
	    llike_diff += cphi-phi;
	  }
	}
      }
    }

    for(int r=nm1r;r<nrater-1;r++){
      llike_diff += .5*(pow(shiftr[r],2)/10.0)-.5*(pow(cshiftr23[r],2)/10.0);
    }

    if(drnunf()<exp(llike_diff)){
      for(int r=nm1r;r<nrater;r++){
	shiftr[r]=cshiftr23[r];
	for(int m=1;m<nmode;m++){
	  for(int k=0;k<maxcat;k++){
	    for(int l=0;l<maxcat;l++){
	      trans[r][m].SetElem(k,l,ctrans23[r][m].GetElem(k,l));
	    }
	  }
	}
      }
    }


    //starting rater scale

    llike_diff=0;
    cscaler23[last]=scaler[last];

    for(int r=nm1r;r<nrater-1;r++){ 

      z=drnnof();
      cscaler23[r]=scaler[r]*exp(scsd[r]*z);
      cscaler23[last]=cscaler23[last]/exp(scsd[r]*z);

      for(int m=0; m<nmode; m++){
	calc_trans(maxcat,shiftr[r],cscaler23[r],shiftm[m],scalem[m],ctrans23[r][m]);
      }  
    }
    for(int m=0; m<nmode; m++){
      calc_trans(maxcat,shiftr[last],cscaler23[last],shiftm[m],scalem[m],ctrans23[last][m]);
    }  

    for(int r=nm1r;r<nrater;r++){    
      for(int j=0;j<nquest;j++){
	for(int i=0;i<nresp;i++){
	  thescore=sc_r[r].GetElem(i,j);
	  if(thescore<item[j].ncat){
	    latresp=xi.GetElem(i,j);
	    trans[r][rimode[r][j]].GetRow(latresp,hold);
	    ctrans23[r][rimode[r][j]].GetRow(latresp,holdc);
	    cumsum(item[j].ncat,hold,ratnorm);
	    cumsum(item[j].ncat,holdc,cratnorm);
	    phi = log(hold[thescore]/ratnorm[item[j].ncat-1]);
	    cphi= log(holdc[thescore]/cratnorm[item[j].ncat-1]);
	    llike_diff += cphi-phi;
	  }
	}
      }
    }

    for(int r=nm1r;r<nrater-1;r++){
      llike_diff += .5*(pow(log(scaler[r]),2)/10.0)-.5*(pow(log(cscaler23[r]),2)/10.0) - log(cscaler23[r]) + log(scaler[r]);
      llike_diff += log( lognorm_dens(scaler[r],log(cscaler23[r]),cscrvar) ) - log( lognorm_dens(cscaler23[r],log(scaler[r]),cscrvar) );
    }

    if(drnunf()<exp(llike_diff)){
      for(int r=nm1r;r<nrater;r++){
	scaler[r]=cscaler23[r];
	for(int m=1;m<nmode;m++){
	  for(int k=0;k<maxcat;k++){
	    for(int l=0;l<maxcat;l++){
	      trans[r][m].SetElem(k,l,ctrans23[r][m].GetElem(k,l));
	    }
	  }
	}
      }
    }

    for(int r=nm1r;r<nrater;r++){
      oMuSir<<shiftr[r]<<" "<<scaler[r]<<" ";             

     if (scaler[r] != prevpsi[r]){
	psijump[r]++;
	prevpsi[r] = scaler[r];
      }
      if (shiftr[r] != prevphi[r]){ 
	phijump[r]++; 
	prevphi[r] = shiftr[r]; 
      } 
    }
    oMuSir<<endl;
  //END UPDATING RATERS
 


   //BEGIN UPDATE MODE !!!
  for(int m=1; m<nmode; m++){

    //begin update mode shift

    llike_diff=0;
    cshiftm=shiftm[m]+mshsd[m]*drnnof();

    for (int j=0; j<nquest; j++){
      for (int r=0; r<nrater; r++){
	if (irmode[j][r] == m){//if r rated j in mode m
	  calc_trans(maxcat,shiftr[r],scaler[r],cshiftm,scalem[m],ctran);
	  for (int i=0; i<nresp; i++){
	    thescore = subj[i].score.GetElem(j,r);
	    if(thescore<item[j].ncat){
	      latresp=xi.GetElem(i,j);
	      trans[r][m].GetRow(latresp,hold);	     
	      ctran.GetRow(latresp,holdc);
	      cumsum(item[j].ncat,hold,ratnorm);
	      cumsum(item[j].ncat,holdc,cratnorm);
	      phi = log(hold[thescore]/ratnorm[item[j].ncat-1]);
	      cphi= log(holdc[thescore]/cratnorm[item[j].ncat-1]);
	      llike_diff += cphi-phi;
	    }
	  }
	}
      }
    }

    llike_diff += .5*(pow(shiftm[m],2)/9)-.5*(pow(cshiftm,2)/9);

    if(drnunf()<exp(llike_diff)){
      shiftm[m]=cshiftm;
      for(int r=0;r<nrater;r++){//update normalizing constants
	calc_trans(maxcat,shiftr[r],scaler[r],shiftm[m],scalem[m],trans[r][m]);
      }
    }

    //begin update mode scale

    llike_diff=0;
    z=drnnof();
    cscalem=scalem[m]*exp(mscsd[m]*z);

    for (int j=0; j<nquest; j++){
      for (int r=0; r<nrater; r++){
	if (irmode[j][r] == m){//if r rated j in mode m
	  calc_trans(maxcat,shiftr[r],scaler[r],shiftm[m],cscalem,ctran);
	  for (int i=0; i<nresp; i++){
	    thescore = subj[i].score.GetElem(j,r);
	    if(thescore<item[j].ncat){
	      latresp=xi.GetElem(i,j);
	      trans[r][m].GetRow(latresp,hold);	     
	      ctran.GetRow(latresp,holdc);
	      cumsum(item[j].ncat,hold,ratnorm);
	      cumsum(item[j].ncat,holdc,cratnorm);
	      phi = log(hold[thescore]/ratnorm[item[j].ncat-1]);
	      cphi= log(holdc[thescore]/cratnorm[item[j].ncat-1]);
	      llike_diff += cphi-phi;
	    }
	  }
	}
      }
    }

    llike_diff += .5*(pow(log(scalem[m]),2)/9)-.5*(pow(log(cscalem),2)/9) - log(cscalem) + log(scalem[m]);
    llike_diff += log( lognorm_dens(scalem[m],log(cscalem),mscsd[m]) ) - log( lognorm_dens(cscalem,log(scalem[m]),mscsd[m]) );

    if(drnunf()<exp(llike_diff)){
      scalem[m]=cscalem;
      for(int r=0;r<nrater;r++){//update normalizing constants
	calc_trans(maxcat,shiftr[r],scaler[r],shiftm[m],scalem[m],trans[r][m]);
      }
    }


      oMode<<shiftm[m]<<" "<<scalem[m]<<" ";

      if (shiftm[m] != prevmphi[m]){
	mphijump[m]++;
	prevmphi[m] = shiftm[m];
      }

      if (scalem[m] != prevmpsi[m]){
	mpsijump[m]++;
	prevmpsi[m] = scalem[m];
      }

    }
    oMode<<endl;
    //END UPDATING MODES




    oTheta<<theta[419]<<" "<<theta[394]<<" "<<theta[285]<<" "
	  <<theta[367]<<" "<<theta[51]<<endl;
  }//


  for (int r=0; r<nrater; r++){ 
    cout << static_cast<float>(phijump[r])/nsim << "  "; 
  }
  cout << endl<<endl;
  for (int r=0; r<nrater; r++){ 
    cout << static_cast<float>(psijump[r])/nsim << "  "; 
  }
  cout << endl<<endl;
  for (int m=1; m<nmode; m++){ 
    cout << static_cast<float>(mphijump[m])/nsim << "  "; 
  } 
  cout << endl<<endl;
  for (int m=1; m<nmode; m++){ 
    cout << static_cast<float>(mpsijump[m])/nsim << "  "; 
  } 
  cout << endl; 

  //write ending time
  time(&time2);
  oLog <<"Program lasted for "<<difftime(time2,time1)/60.0<<" minutes and ended at "<< ctime(&time2);

  return(0);
}





















