//These functions comprise a portion of the Metropolis-Hastings updates 
//utilized in the mode.hrm.cpp

//edited from hrm.update.cpp by Lou Mariano (2000,2001,2002)

#include<stdlib.h>
#include<math.h>
#include<iostream.h>
#include "PCM.hpp"

#define drnunf() drand48()
#define drnnof() rnorm()


double lognorm_dens(double x, double mean, double sd);
double rgamma(double);
double rnorm(void);
double prob_calc(Quest task,double theta,int x);
void calc_trans(int ncat,double mur,double sigmar,double mum,double sigmam,
		Mat<double> &trans);
void alt_calc_trans(int ncat,double mur,double sigmar,double mum,double sigmam,
		Mat<double> &trans);
void pcm (Quest task,double theta,double prob[]);
void cumsum(int dim,double gamma[],double eta[]);


//#########################################################################

void update_beta_cons(int nresp,Quest &task,Quest &taskc
		 ,double theta[],int x[],int xc[]){
  double prob[task.ncat],cprob[task.ncat],probc[task.ncat];
  double cprobc[task.ncat],lprior,clprior,lrat,u;
  double llike=0,cllike=0,acc=0;
  Quest cand=task;
  Quest candc=taskc;
  double t;
  t=drnnof();
  double cvar=.15;
  cand.SetBeta(task.beta+cvar*t);
  candc.SetBeta(taskc.beta-cvar*t);

  for(int i=0;i<nresp;i++){
    if(x[i]==task.ncat){
      llike+=0;
      cllike+=0;
    }
    else{
      pcm(task,theta[i],prob);
      pcm(taskc,theta[i],probc);
      pcm(cand,theta[i],cprob);
      pcm(candc,theta[i],cprobc);
      llike += log(prob[x[i]])+log(probc[xc[i]]);
      cllike += log(cprob[x[i]])+log(cprobc[xc[i]]);
    }
  }
  lprior=0-.5*(task.beta * task.beta)/10;
  clprior=0-.5*(cand.beta * cand.beta)/10;
  lrat=cllike+clprior-lprior-llike;
  u=drnunf();
  if(u < exp(lrat)){
    task=cand;
    taskc=candc;
  }
  return;
}

//#######################################################################

void update_gamma(int nresp,Quest &task,double theta[],int x[]){
  double prob[task.ncat],cprob[task.ncat];
  double lprior=0,clprior=0,lrat=0,u;
  double llike=0,cllike=0;
  double t;
  Quest cand;
  
  cand.SetNcat(task.ncat);
  cand.SetBeta(task.beta);
  cand.SetGamma(task.gamma);
  
  for(int j=0;j<task.ncat-2;j++){
    t=drnnof();
    double cvar=.2;
    cand.gamma[j] = task.gamma[j]+cvar*t;
    cand.gamma[task.ncat-2] = task.gamma[task.ncat-2]-cvar*t;
    for(int i=0;i<nresp;i++){
      if(x[i]==task.ncat){
	llike+=0;
	cllike+=0;
      }
      else{
	pcm(task,theta[i],prob);
	pcm(cand,theta[i],cprob);
	llike += log(prob[x[i]]);
	cllike += log(cprob[x[i]]);
      }
    }
    lprior=0-.5*(task.gamma[j] * task.gamma[j])/3;
    clprior=0-.5*(cand.gamma[j] * cand.gamma[j])/3;
    lrat=cllike+clprior-llike-lprior;
    
    u=drnunf();
    if(u < exp(lrat)){
      task.gamma[j]=cand.gamma[j];
      task.gamma[task.ncat-2]=cand.gamma[task.ncat-2];
    }
  }
  cand.FreeNcat();
  return;
}

//####################################################################

void update_var(int nresp, double theta[], double &mu,double &tau){
  //updates both the var (tau) and mean (mu) of the ability params
  //tau drawn from inv.gamma(alpha,beta)
  //mu drawn from N(mn,sd^2)
  //nresp is number of examinees
  //theta[] are the current ability params
  double z,alpha=1,beta=1,xbar=0,sd,mn,pvar=10;
  //z is temp var for a N(0,1) draw
  //alpha,beta are inv.gamma shape and scale
  //alpha=1,beta=1 are the prior values
  //xbar will be mean of current thetas
  //mn and sd are hyperparams for mu when drawn
  //pvar is prior variance of mu
  int nr=1;
  double ss=0,scale,chi[nr];
  //ss will be sum of (theta[i]-mu)^2
 for(int j=0;j<nresp;j++){
    xbar+=theta[j]/nresp;

    ss += (theta[j]-mu) * (theta[j]-mu);

  } //calc xbar and ss
  alpha += nresp/2.; //update posterior inv.gamma shape
  beta += ss/2.; //update posterior inv.gamma scale
  chi[0]=rgamma(alpha); //draw gamma(alpha,1)
  tau=beta/(chi[0]); //convert to inv.gamma(alpha,beta)
  z=drnnof(); //draw N(0,1)
  mn=nresp*pvar*xbar/(nresp*pvar+tau); //update posterior mn
  sd=sqrt(tau*pvar/(nresp*pvar+tau)); //update posterior sd
  mu=mn+z*sd; //convert to N(mn,sd^2)
  return;
}


//##############################################################3

double prob_calc(Quest task,double theta,int x){
  double prob[task.ncat];
  pcm(task,theta,prob);
  return(prob[x]);
}

//###################################################################
void update_theta(int nquest,Quest task[],double &theta,
		  double mu,double tau,int x[]){

  
  double prob,cprob;
  double cth,u;
  u=drnnof();
  double cvar=1.5;
  cth=theta+cvar*u;
  double llike=0,lprior;
  double cllike=0,clprior,lrat;
  for(int j=0;j<nquest;j++){
    if(x[j]==task[j].ncat){
      llike+=0;
      cllike+=0;
    }
    else{
      prob=prob_calc(task[j],theta,x[j]); 
      // Calculate probability for old theta     
      cprob=prob_calc(task[j],cth,x[j]);  // Calcuate prob for candidate theta
      llike += log(prob);                 // Log-Like for old theta
      cllike += log(cprob);               // Log-Like for cand theta
    }
  }
  
  lprior= 0 - .5 * ( (theta-mu) * (theta-mu) ) / tau;
  clprior= 0 - .5 * ( (cth-mu) * (cth-mu) ) / tau;
  lrat=cllike+clprior-llike-lprior;
  u=drnunf();
  if(u < exp(lrat)){
    theta=cth;
  }
  return;
}


//#################################################################


void calc_trans(int ncat,double mur,double sigmar,double mum,double sigmam,Mat<double> &trans){

  /*This function calculates a term used in the denominator of complete 
    conditional posteriors of the ideal scores and the parameters 
    governing the signal detection probabilities*/
  
  //mur,sigmar are the contributions to severity and consistency by the rater
  //mum,sigmam are the contributions to severity and consistency by the mode

  double z;
  double q,p;
  double sum;//to verify row nonzero row-sums
  bool zerosum=false;//is there a zero sum row in trans  
  
  
  for(int l=0;l<ncat;l++){//cycle through ideal scores
    sum =0;//initialize row-sum to zero
    for(int k=0;k<ncat;k++){//cycle through categories
      z=(k-l-mur-mum)/(sigmar*sigmam);
      q = 0 - (z * z) /2;
      p=exp(q);
      trans.SetElem(l,k,p);
      sum += p;
    }
    if (sum == 0){
      zerosum=true;
    }    
  }

  if (zerosum){
  alt_calc_trans(ncat, mur, sigmar, mum, sigmam, trans);
  }

}

void alt_calc_trans(int ncat,double mur,double sigmar,double mum,double sigmam,Mat<double> &trans){


  double z[ncat];
  double q[ncat],p[ncat];


  for(int l=0;l<ncat;l++){//cycle through ideal scores
    for(int k=0;k<ncat;k++){//cycle through categories
      z[k]=(k-l-mur-mum);
    }
    for(int k=0;k<ncat;k++){//cycle through categories
      p[k]=0;
      for(int h=0;h<ncat;h++){
	q[h]= pow(z[h],2) - pow(z[k],2);
	p[k]+= exp( -.5 * q[h] / ( pow(sigmar,2) * pow(sigmam,2) ) );
      }
      trans.SetElem(l,k,1/p[k]);
    }
  }
}




//#####################################################################

double lognorm_dens(double x, double mean, double sd){
  //This function will calculate the density under a log-normal pdf.

  //When used to calc. the density of a draw from a log-normal, where the draw was taken from a N(0,1) and transformed, remember to submit the "log(mean)" the log of the mean of the draw from the normal dist.

  double hold, var;

  var = pow(sd,2);
  hold = -.5 * pow( ( log(x) - mean ) , 2) / var;
  hold = exp(hold);

  const double PI=3.14159265358979323;
  const double marg=sqrt( 2 * PI );

  hold = hold/(x * marg * sd);

  return(hold);
}
