//contains functions for generating random variables needed in hrp.cpp

//functions courtesy of J.R. Lockwood

//These functions replace those used in Matt Johnson's original HRM code 
//as to not rely on a seperate commercial library. 

#include <stdlib.h>
#include <math.h>
#include <iostream.h>
#include <time.h>

double genrand_real3();             //***RNG***
#define drand48() genrand_real3()   //***RNG***

#define E 2.718281812459045




double runif(double lower, double upper)
{
  return (upper-lower)*drand48() + lower;
}


int rdisc(double *ptr, int length)
{
        int len;
        double sum;
        double unif;

        sum=0.0;
        for (len=0;len<length;len++) sum+=*(ptr+len);

        unif = runif(0,sum);

        len=0;
        sum=*(ptr+len);
        while(sum<unif) { len++; sum+=*(ptr+len); }
        return len;
}



double rnorm(void)
{

  static int iset=0;
  static double gset;
  double fac, rsq, v1, v2;

  if(iset==0) {
    do {
      v1 = 2.0*drand48()-1.0;
      v2 = 2.0*drand48()-1.0;
      rsq = v1*v1 + v2*v2;
    } 
    while (rsq >= 1.0 ||rsq == 0.0);
    fac = sqrt (-2.0*log(rsq)/rsq);
    gset=v1*fac;
    iset=1;
    return v2*fac;
  } 
  else{
    iset=0;
    return gset;
  }
}


double rgamma(double aa)
  /*
    This subrountine is from Mike West's code.  JR slightly modified it
    to use drand48 as the uniform random number generator.
    Generates a random gamma variable with shape aa>0 and scale=1
  */

{
  double ea,u0,u1,c1,c2,c3,c4,c5,w,gamdv;
  
  /* if (aa <= 0.0) return -1.0;  error: shape parameter must be >0 */
  
  if ( aa == 1.0 ) return (-1.0 * log(drand48()));
  /* exponential */
  
  if ( aa <= 1.0 ) {
    ea = E / ( aa + E );
    while (1) {
      u0 = drand48();
      u1 = drand48();
      if ( u0 <= ea ) {
	gamdv = pow( u0 / ea , 1.0 / aa );
	if ( u1 > exp( -1.0 * gamdv ) ) {
	  continue;
	} 
	else {
	  return gamdv;
	}
      } 
      else {
	/*if ((1.0-u0)/(ea*aa) <= 0) 
	  fprintf(stderr,"rgamma: %lf\n",(1.0-u0)/(ea*aa));*/
	gamdv = -1.0 * log( (1.0-u0)/(ea*aa));
	if ( u1 > pow(gamdv, aa - 1.0) ) {
	  continue;
	} 
	else {
	  return gamdv;
	}
      }
    }
  } 
  else { 
    /* aa > 1.0 */
    c1 = aa - 1.0;
    c2 = ( aa- 1.0/(6.0*aa) ) / c1;
    c3 = 2.0 / c1;
    c4 = c3 + 2.0;
    c5 = 1.0 / sqrt(aa);
    while (1) {
      u1 = drand48();
      u0 = drand48();
      u1 = u0 + c5 * ( 1.0-1.86 * u1 );
      if ( fabs( u1- 0.5 ) >= 0.5 ) {
	continue;
      }
      w = c2 * u0 / u1;
      /*if (u1<=0) fprintf(stderr,"rgamma: u1=%lf\n",u1);*/
      /*if (w<=0) fprintf(stderr,"rgamma: w=%lf\n",w);*/
      if ( ( c3*log( u1 ) - log( w ) + w) >= 1.0 ) {
	continue;
      } 
      else {
	return (c1 * w);
      }
    }
  }
}





